/**
 * Program PhidgetSensor.java
 * 
 * Date         Version    Author         Comment
 * 2008/03/18   1.0.0      T.Kawashima    First.
 * 
 */

package robomin.sensor;

import java.util.logging.Level;

import com.phidgets.InterfaceKitPhidget;
import com.phidgets.Phidget;
import com.phidgets.PhidgetException;
import com.phidgets.event.AttachListener;
import com.phidgets.event.DetachListener;
import com.phidgets.event.ErrorListener;
import com.phidgets.event.InputChangeListener;
import com.phidgets.event.OutputChangeListener;
import com.phidgets.event.SensorChangeListener;

import robomin.util.RobominDef;
import robomin.util.RobominInfo;
import robomin.util.RobominLogger;

/**
 * PhidgetZT[̒l擾NX
 * @author T.Kawashima
 * @version 1.0.0
 */
public class PhidgetIKP {
	private RobominLogger logger;
	private InterfaceKitPhidget ikp;
	
	/**
	 * RXgN^
	 * @param info RobominInfo
	 */
	public PhidgetIKP(RobominInfo info) {
		logger = info.getRobominLogger();
		
		logger.log(Level.INFO, "Load Phidget library:" + Phidget.getLibraryVersion());
		
		try {
			ikp = new InterfaceKitPhidget();
		}
		catch (PhidgetException e) {
			logger.log(Level.SEVERE, e.toString());
		}
	}
	
	/**
	 * Phidgetɐڑ^CAEg܂ő҂
	 * @param info PhidgetInfo
	 * @param timeout ~sec
	 * @return ڑ0Aڑs<0
	 */
	public int waitForAttachment(PhidgetInfo info, int timeout) {
		return waitForAttachment(info.getSerialNum(), info.getIpAddr(), info.getPortNum(), info.getPasswd(), timeout);
	}
	
	/**
	 * Phidgetɐڑ^CAEg܂ő҂
	 * @param serial VAԍ
	 * @param ipAddr IPAhX
	 * @param port |[gԍ
	 * @param passwd pX[h(ȗ)
	 * @param timeout ^CAEg
	 * @return ڑ0Aڑs<0
	 */
	public int waitForAttachment(int serial, String ipAddr, int port, String passwd, int timeout) {
		try {
			ikp.open(serial, ipAddr, port, passwd);
			ikp.waitForAttachment(timeout);
			logger.log(Level.INFO, "Remote Phidget InterfaceKit opend ("  + serial + ") " + ipAddr + ":" + port);
		}
		catch (PhidgetException e) {
			logger.log(Level.SEVERE, e.toString());
			return RobominDef.FAIL;
		}
		
		return RobominDef.SUCCESS;
	}
	
	/**
	 * [JPhidgetɐڑ^CAEg܂ő҂
	 * @param timeout ^CAEg
	 * @return ڑ0Aڑs<0
	 */
	public int waitForAttachment(int timeout) {
		try {
			ikp.openAny();
			ikp.waitForAttachment(timeout);
			logger.log(Level.INFO, "Local Phidget InterfaceKit opened");
		}
		catch (PhidgetException e) {
			logger.log(Level.SEVERE, e.toString());
			return RobominDef.FAIL;
		}
		
		return RobominDef.SUCCESS;
	}
	
	/**
	 * Phidget̐ڑ
	 * @return ڑ0Aڑs<0
	 */
	public int close() {
		try {
			ikp.close();
			logger.log(Level.INFO, "Phidget InterfaceKit closed");
		}
		catch (PhidgetException e) {
			logger.log(Level.SEVERE, e.toString());
			return RobominDef.FAIL;
		}
		
		return RobominDef.SUCCESS;
	}
	
	/**
	 * Xi[o^
	 * @param listener o^郊Xi[
	 */
	public void addAttachListener(AttachListener listener) {
		if (ikp != null) {
			ikp.addAttachListener(listener);
		}
	}
	
	/**
	 * Xi[o^
	 * @param listener o^郊Xi[
	 */
	public void addDetachListener(DetachListener listener) {
		if (ikp != null) {
			ikp.addDetachListener(listener);
		}
	}
	
	/**
	 * Xi[o^
	 * @param listener o^郊Xi[
	 */
	public void addErrorListener(ErrorListener listener) {
		if (ikp != null) {
			ikp.addErrorListener(listener);
		}
	}
	
	/**
	 * Xi[o^
	 * @param listener o^郊Xi[
	 */
	public void addInputChangeListener(InputChangeListener listener) {
		if (ikp != null) {
			ikp.addInputChangeListener(listener);
		}
	}
	
	/**
	 * Xi[o^
	 * @param listener o^郊Xi[
	 */
	public void addOutputChangeListener(OutputChangeListener listener) {
		if (ikp != null) {
			ikp.addOutputChangeListener(listener);
		}
	}
	
	/**
	 * Xi[o^
	 * @param listener o^郊Xi[
	 */
	public void addSensorChangeListener(SensorChangeListener listener) {
		if (ikp != null) {
			ikp.addSensorChangeListener(listener);
		}
	}
	
	/**
	 * PhidgetZT[f[^CfbNXl𔲂oԂ
	 * @param str PhidgetZT[f[^
	 * @return CfbNXl
	 */
	public int getSensorIndex(String str) {
		int sensorIndex = str.indexOf("sensor");
		int sensorFast = str.indexOf(" ", sensorIndex);
		int sensorLast = str.indexOf(" ", sensorFast+1);
		
		return Integer.parseInt(str.substring(sensorFast+1, sensorLast));
	}
	
	/**
	 * PhidgetZT[f[^ZT[l𔲂oԂ
	 * @param str PhidgetZT[f[^
	 * @return ZT[l
	 */
	public int getSensorValue(String str) {
		int valueFast = str.lastIndexOf(" ");
		int valueLast = str.length();
		
		return Integer.parseInt(str.substring(valueFast+1, valueLast));
	}
}
