/**
 * Program PhidgetWizard.java
 * 
 * Date         Version    Author         Comment
 * 2008/03/19   1.0.0      T.Kawashima    First.
 * 
 */

package robomin.ui;

import java.util.logging.Level;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import robomin.sensor.PhidgetDef;
import robomin.util.RobominDef;
import robomin.util.RobominInfo;
import robomin.util.RobominLogger;

/**
 * Phidgetڑ̃EBU[h
 * @author T.Kawashima
 * @version 1.0.0
 */
public class PhidgetWizard extends Wizard {
	private int language = 0;
	private RobominLogger logger;
	private PhidgetConnectPage cPage;
	private PhidgetSensorPage iPage;
	public int phidgetDest;
	public int serialNumber;
	public String ipAddr;
	public int portNumber;
	public String passwd;
	public int joyXIO;
	public int joyYIO;

	/**
	 * RXgN^
	 * @param info RobominInfo
	 */
	public PhidgetWizard(RobominInfo info) {
		super();
		this.language = info.getLanguage();
		this.logger = info.getRobominLogger();
		setWindowTitle(GUIDef.WIZARD_PHIDGET_TITLE[language]);
		
		cPage = new PhidgetConnectPage();
		iPage = new PhidgetSensorPage();
	}
	
	/**
	 * Finish{^ꂽƂɌĂ΂
	 */
	public boolean performFinish() {
		phidgetDest = cPage.getDest();
		serialNumber = cPage.getSerial();
		ipAddr = cPage.getAddr();
		portNumber = cPage.getPort();
		passwd = cPage.getPasswd();
		joyXIO = iPage.getJoyXIO();
		joyYIO = iPage.getJoyYIO();
		
		logger.log(Level.INFO, "Wizard finished");
		
		return true;
	}
	
	public boolean performCancel() {
		logger.log(Level.INFO, "Wizard canceled");
		return true;
	}
	
	/**
	 * EBU[hɃy[Wǉ
	 */
	public void addPages() {
		addPage(cPage);
		addPage(iPage);
	}

	/**
	 * ڑIy[W
	 * @author T.Kawashima
	 * @version 1.0.0
	 */
	class PhidgetConnectPage extends WizardPage implements ModifyListener {
		int phidgetDest = PhidgetDef.DEST_LOCAL;
		Button localBtn;
		Button remoteBtn;
		Text serialText;
		Text ipAddrText;
		Text portText;
		Text passwdText;
		private Pattern idPattern1;
		private Pattern idPattern2;
		
		/**
		 * RXgN^
		 */
		public PhidgetConnectPage() {
			super(GUIDef.WIZARD_PHIDGET_P1_TITLE[language]);
			setTitle(GUIDef.WIZARD_PHIDGET_P1_TITLE[language]);
			idPattern1 = Pattern.compile("[0-9]{5}");
			idPattern2 = Pattern.compile("[0-9]{1,}");
		}
		
		/**
		 * GUI𐶐
		 */
		public void createControl(Composite parent) {
            Composite c = new Composite(parent, SWT.NONE);
            c.setLayout(new GridLayout(2, false));
            
            localBtn = new Button(c, SWT.RADIO);
            localBtn.setText(GUIDef.WIZARD_PHIDGET_P1_BTN1[language]);
            localBtn.setSelection(true);
            localBtn.addSelectionListener(new SelectionAdapter() {
            	public void widgetSelected(SelectionEvent e) {
            		if (((Button)e.widget).getSelection()) {
            			phidgetDest = PhidgetDef.DEST_LOCAL;
            			ipAddrText.setEnabled(false);
            			portText.setEnabled(false);
            			passwdText.setEnabled(false);
            			updateStatus(null);
            		}
            	}
            });
            
            remoteBtn = new Button(c, SWT.RADIO);
            remoteBtn.setText(GUIDef.WIZARD_PHIDGET_P1_BTN2[language]);
            remoteBtn.addSelectionListener(new SelectionAdapter() {
            	public void widgetSelected(SelectionEvent e) {
            		if (((Button)e.widget).getSelection()) {
            			phidgetDest = PhidgetDef.DEST_REMOTE;
            			ipAddrText.setEnabled(true);
            			portText.setEnabled(true);
            			passwdText.setEnabled(true);
            			modifyText(null);
            		}
            	}
            });
            
            // \̒
            new Label(c, SWT.NONE).setText("");
            new Label(c, SWT.NONE).setText("");
            
            new Label(c, SWT.NONE).setText(GUIDef.WIZARD_PHIDGET_P1_LBL1[language]);
            serialText = new Text(c, SWT.BORDER | SWT.SINGLE);
            serialText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
            serialText.addModifyListener(this);
            
            new Label(c, SWT.NONE).setText(GUIDef.WIZARD_PHIDGET_P1_LBL2[language]);
            ipAddrText = new Text(c, SWT.BORDER | SWT.SINGLE);
            ipAddrText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
            ipAddrText.setEnabled(false);
            ipAddrText.addModifyListener(this);
            
            new Label(c, SWT.NONE).setText(GUIDef.WIZARD_PHIDGET_P1_LBL3[language]);
            portText = new Text(c, SWT.BORDER | SWT.SINGLE);
            portText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
            portText.setEnabled(false);
            portText.addModifyListener(this);
            
            new Label(c, SWT.NONE).setText(GUIDef.WIZARD_PHIDGET_P1_LBL4[language]);
            passwdText = new Text(c, SWT.BORDER | SWT.SINGLE | SWT.PASSWORD);
            passwdText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
            passwdText.setEnabled(false);
            
            setControl(c);
		}
		
		/**
		 * ͂ꂽڑԂ
		 * @return
		 */
		public int getDest() {
			return this.phidgetDest;
		}
		
		/**
		 * ͂ꂽVAԍԂ
		 * @return VAԍ
		 */
		public int getSerial() {
			try {
				return Integer.parseInt(this.serialText.getText());
			}
			catch (NumberFormatException e) {
				return RobominDef.FAIL;
			}
		}
		
		/**
		 * ͂ꂽIPAhXԂ
		 * @return IPAhX
		 */
		public String getAddr() {
			return this.ipAddrText.getText();
		}
		
		/**
		 * ͂ꂽ|[gԍԂ
		 * @return |[gԍ
		 */
		public int getPort() {
			try {
				return Integer.parseInt(this.portText.getText());
			}
			catch (NumberFormatException e) {
				return RobominDef.FAIL;
			}
		}
		
		/**
		 * ͂ꂽpX[hԂ
		 * @return pX[h
		 */
		public String getPasswd() {
			return this.passwdText.getText();
		}
		
		/**
		 * ̓tB[hύXꂽɌĂ΂
		 */
		public void modifyText(ModifyEvent e) {
            Matcher m1 = idPattern1.matcher(String.valueOf(getSerial()));
            Matcher m2 = idPattern2.matcher(String.valueOf(getPort()));
            
            if (remoteBtn.getSelection()) {
	            if (!m1.matches()){
	                    updateStatus(GUIDef.WIZARD_PHIDGET_P1_ERR1[language]);
	                    return;
	            }
            
	            if (getAddr().length()==0){
	                    updateStatus(GUIDef.WIZARD_PHIDGET_P1_ERR2[language]);
	                    return;
	            }
	            
	            if (!m2.matches()){
                    updateStatus(GUIDef.WIZARD_PHIDGET_P1_ERR3[language]);
                    return;
	            }
            }

            updateStatus(null);
		}

		
		/**
		 * EBU[h̃bZ[Wƃ{^̏ԂύX
		 * @param str \郁bZ[W
		 */
        private void updateStatus(String str){
            setErrorMessage(str);
            setPageComplete(str == null);
        }

	}
	
	/**
	 * ZT[Iy[W
	 * @author T.Kawashima
	 * @version 1.0.0
	 */
	class PhidgetSensorPage extends WizardPage implements ModifyListener {
		private Button interfaceBtn;
		private Button rfidBtn;
		private Button accelBtn;
		private Label[] labels;
		private Combo[] combos;
		
		/**
		 * RXgN^
		 */
		public PhidgetSensorPage() {
			super(GUIDef.WIZARD_PHIDGET_P2_TITLE[language]);
			setTitle(GUIDef.WIZARD_PHIDGET_P2_TITLE[language]);
			
			labels = new Label[PhidgetDef.IOPORT_NUM];
			combos = new Combo[PhidgetDef.IOPORT_NUM];
		}
		
		/**
		 * GUI𐶐
		 */
		public void createControl(Composite parent) {
            Composite c = new Composite(parent, SWT.NONE);
            c.setLayout(new GridLayout(2, false));
            
            interfaceBtn = new Button(c, SWT.RADIO);
            interfaceBtn.setText(GUIDef.WIZARD_PHIDGET_P2_S1[language]);
            interfaceBtn.setSelection(true);
            interfaceBtn.addSelectionListener(new SelectionAdapter() {
            	public void widgetSelected(SelectionEvent e) {
            		if (((Button)e.widget).getSelection()) {
            			combosSetStatus(true);
            		}
            	}
            });

            rfidBtn = new Button(c, SWT.RADIO);
            rfidBtn.setText(GUIDef.WIZARD_PHIDGET_P2_S2[language]);
            rfidBtn.addSelectionListener(new SelectionAdapter() {
            	public void widgetSelected(SelectionEvent e) {
            		if (((Button)e.widget).getSelection()) {
            			combosSetStatus(false);
            		}
            	}
            });
            
            accelBtn = new Button(c, SWT.RADIO);
            accelBtn.setText(GUIDef.WIZARD_PHIDGET_P2_S3[language]);
            accelBtn.addSelectionListener(new SelectionAdapter() {
            	public void widgetSelected(SelectionEvent e) {
            		if (((Button)e.widget).getSelection()) {
            			combosSetStatus(false);
            		}
            	}
            });

            // \̒
            new Label(c, SWT.NONE).setText("");
            new Label(c, SWT.NONE).setText("");
            new Label(c, SWT.NONE).setText("");
            
            new Label(c, SWT.NONE).setText(GUIDef.WIZARD_PHIDGET_P2_LBL1[language]);
            new Label(c, SWT.NONE).setText(GUIDef.WIZARD_PHIDGET_P2_LBL2[language]);
            
            for (int i=0; i<PhidgetDef.IOPORT_NUM; i++) {
            	labels[i] = new Label(c, SWT.NONE);
            	combos[i] = new Combo(c, SWT.BORDER | SWT.DROP_DOWN);
                combos[i].setText(i+"");
                combos[i].add("0");
                combos[i].add("1");
                combos[i].add("2");
                combos[i].add("3");
                combos[i].add("4");
                combos[i].add("5");
                combos[i].add("6");
                combos[i].add("7");
                combos[i].select(i);
                combos[i].setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
                combos[i].addModifyListener(this);
            }

            labels[0].setText(GUIDef.WIZARD_PHIDGET_P2_LBL3[language]);
            labels[1].setText(GUIDef.WIZARD_PHIDGET_P2_LBL4[language]);
            labels[2].setText(GUIDef.WIZARD_PHIDGET_P2_LBL5[language]);
            labels[3].setText(GUIDef.WIZARD_PHIDGET_P2_LBL6[language]);
            labels[4].setText(GUIDef.WIZARD_PHIDGET_P2_LBL7[language]);
            labels[5].setText(GUIDef.WIZARD_PHIDGET_P2_LBL8[language]);
            labels[6].setText(GUIDef.WIZARD_PHIDGET_P2_LBL9[language]);
            labels[7].setText(GUIDef.WIZARD_PHIDGET_P2_LBL10[language]);
            
            setControl(c);
		}
		
		/**
		 * ̓tB[hύXꂽɌĂ΂
		 */
		public void modifyText(ModifyEvent e) {
            if (interfaceBtn.getSelection()) {
            	// I/O|[g͕sƂG[\
            	for (int i=0; i<PhidgetDef.IOPORT_NUM; i++) {
            		for (int j=0; j<PhidgetDef.IOPORT_NUM; j++) {
            			if (i==j) continue;
            			if (combos[i].getText().equals(combos[j].getText())) {
    	                    updateStatus(GUIDef.WIZARD_PHIDGET_P2_ERR1[language] + "\n("  + labels[i].getText() + ") and (" + labels[j].getText() + ")");
    	                    return;
            			}
            		}
            	}
            }

            updateStatus(null);
		}

		
		/**
		 * EBU[h̃bZ[Wƃ{^̏ԂύX
		 * @param str
		 */
        private void updateStatus(String str){
            setErrorMessage(str);
            setPageComplete(str == null);
        }
        
        /**
         * CombȍԂ܂Ƃ߂ĕύX
         * @param isEnable ύX
         */
        private void combosSetStatus(boolean isEnable) {
        	for (int i=0; i<PhidgetDef.IOPORT_NUM; i++) {
        		combos[i].setEnabled(isEnable);
        	}
        }
        
        /**
         * WCXeBbNXlԂ
         * @return Xl
         */
        public int getJoyXIO() {
        	return this.combos[0].getSelectionIndex();
        }

        /**
         * WCXeBbNYlԂ
         * @return Yl
         */
        public int getJoyYIO() {
        	return this.combos[1].getSelectionIndex();
        }
	}
}
